unit uDiscoverDifferencesDialog;
{$INCLUDE 'DirSync3.inc'}

interface

uses
  SysUtils, Windows, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uSystem,
  uDirSyncConsts, uMessages, uOptions;

type
  TDiscoverDifferencesDialog = class(TFarDialog)
  private
    fLeftPanel: TFarPanel;
    fRightPanel: TFarPanel;
    fOptions: TDirSyncOptions;
    fFrame: TFarDialog.TDoubleBox;
    fLabelTitle: TFarDialog.TLabel;
    fCheckProcessSubfolders: TFarDialog.TCheckbox;
    fCheckUseMaxScanDepth: TFarDialog.TCheckbox;
    fEditMaxScanDepth: TFarDialog.TEdit;
    fCheckProcessSelected: TFarDialog.TCheckbox;
    fSeparator1: TFarDialog.TSeparator;
    fLabelCompareBox: TFarDialog.TLabel;
    fCheckCompareTime: TFarDialog.TCheckbox;
    fCheckCompareLowPrecision: TFarDialog.TCheckbox;
    fCheckIgnoreTimeZone: TFarDialog.TCheckbox;
    fCheckCompareSize: TFarDialog.TCheckbox;
    fCheckCompareContents: TFarDialog.TCheckbox;
    fCheckCompareContentsIgnore: TFarDialog.TCheckbox;
    fRadioIgnoreNewLines: TFarDialog.TRadioButton;
    fRadioIgnoreWhiteSpace: TFarDialog.TRadioButton;
    fSeparator2: TFarDialog.TSeparator;
    fCheckSelectOnBothPanels: TFarDialog.TCheckbox;
    fSeparator3: TFarDialog.TSeparator;
    fButtonOK: TFarDialog.TButton;
    fButtonCancel: TFarDialog.TButton;
    fWidth: integer;
    fHeight: integer;
  protected
    function DialogProc(Msg, Param1, Param2: TIntPtr; var DlgProcResult: TIntPtr): boolean; override;
    procedure OptionsToDialog;
    procedure DialogToOptions;
    procedure UpdateControls(ActiveCheckbox: TFarDialog.TCheckbox; ActiveCheckboxFinalState: TFarDialog.TCheckbox.TCheckState);
    procedure CheckboxClicked(Sender: TFarDialog.TCheckbox; FinalState: TFarDialog.TCheckbox.TCheckState; var AllowChange: boolean);
    property Frame: TFarDialog.TDoubleBox read fFrame;
    property LabelTitle: TFarDialog.TLabel read fLabelTitle;
    property CheckProcessSubfolders: TFarDialog.TCheckbox read fCheckProcessSubfolders;
    property CheckUseMaxScanDepth: TFarDialog.TCheckbox read fCheckUseMaxScanDepth;
    property EditMaxScanDepth: TFarDialog.TEdit read fEditMaxScanDepth;
    property CheckProcessSelected: TFarDialog.TCheckbox read fCheckProcessSelected;
    property Separator1: TFarDialog.TSeparator read fSeparator1;
    property LabelCompareBox: TFarDialog.TLabel read fLabelCompareBox;
    property CheckCompareTime: TFarDialog.TCheckbox read fCheckCompareTime;
    property CheckCompareLowPrecision: TFarDialog.TCheckbox read fCheckCompareLowPrecision;
    property CheckIgnoreTimeZone: TFarDialog.TCheckbox read fCheckIgnoreTimeZone;
    property CheckCompareSize: TFarDialog.TCheckbox read fCheckCompareSize;
    property CheckCompareContents: TFarDialog.TCheckbox read fCheckCompareContents;
    property CheckCompareContentsIgnore: TFarDialog.TCheckbox read fCheckCompareContentsIgnore;
    property RadioIgnoreNewLines: TFarDialog.TRadioButton read fRadioIgnoreNewLines;
    property RadioIgnoreWhiteSpace: TFarDialog.TRadioButton read fRadioIgnoreWhiteSpace;
    property Separator2: TFarDialog.TSeparator read fSeparator2;
    property CheckSelectOnBothPanels: TFarDialog.TCheckbox read fCheckSelectOnBothPanels;
    property Separator3: TFarDialog.TSeparator read fSeparator3;
    property ButtonOK: TFarDialog.TButton read fButtonOK;
    property ButtonCancel: TFarDialog.TButton read fButtonCancel;
    property Width: integer read fWidth;
    property Height: integer read fHeight;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; override;
    function Execute: boolean;
    property LeftPanel: TFarPanel read fLeftPanel write fLeftPanel;
    property RightPanel: TFarPanel read fRightPanel write fRightPanel;
    property Options: TDirSyncOptions read fOptions write fOptions;
  end;

implementation

{ TDirSyncMainDialog }

procedure TDiscoverDifferencesDialog.CheckboxClicked(Sender: TFarDialog.TCheckbox; FinalState: TFarDialog.TCheckbox.TCheckState; var AllowChange: boolean);
begin
  UpdateControls(Sender, FinalState);
end;

procedure TDiscoverDifferencesDialog.Clear;
begin
  inherited;
  fFrame := nil;
  fLabelTitle := nil;
  fCheckProcessSubfolders := nil;
  fCheckUseMaxScanDepth := nil;
  fEditMaxScanDepth := nil;
  fCheckProcessSelected := nil;
  fSeparator1 := nil;
  fLabelCompareBox := nil;
  fCheckCompareTime := nil;
  fCheckCompareLowPrecision := nil;
  fCheckIgnoreTimeZone := nil;
  fCheckCompareSize := nil;
  fCheckCompareContents := nil;
  fCheckCompareContentsIgnore := nil;
  fRadioIgnoreNewLines := nil;
  fRadioIgnoreWhiteSpace := nil;
  fSeparator2 := nil;
  fCheckSelectOnBothPanels := nil;
  fSeparator3 := nil;
  fButtonOK := nil;
  fButtonCancel := nil;
end;

constructor TDiscoverDifferencesDialog.Create;
begin
  inherited Create(DIALOG_MAIN_GUID);
  fWidth := 66;
  fHeight := 22;
  fFrame := AddDoubleBox(3, 1, fWidth-6, fHeight-2, GetMsg(MTitle));
  fLabelTitle := AddLabel(5, 2, 0, GetMsg(MProcessBox));
  fCheckProcessSubfolders := AddCheckbox(5, 3, GetMsg(MProcessSubfolders), False);
  fCheckProcessSubfolders.OnClick := CheckboxClicked;
  fCheckUseMaxScanDepth := AddCheckbox(9, 4, GetMsg(MUseMaxScanDepth), False);
  fCheckUseMaxScanDepth.OnClick := CheckboxClicked;
  fEditMaxScanDepth := AddEdit(fCheckUseMaxScanDepth.Left + Length(fCheckUseMaxScanDepth.Text) + 5, fCheckUseMaxScanDepth.Top, 5, '99');
  fEditMaxScanDepth.Mask := '99999';
  fEditMaxScanDepth.MaxLength := Length(fEditMaxScanDepth.Mask);
  fEditMaxScanDepth.Width := fEditMaxScanDepth.MaxLength + 1;
  fCheckProcessSelected := AddCheckbox(5, 5, GetMsg(MProcessSelected), False);
  fSeparator1 := AddSeparator(6);
  fLabelCompareBox := AddLabel(5, 7, 0, GetMsg(MCompareBox));
  fCheckCompareTime := AddCheckbox(5, 8, GetMsg(MCompareTime), False);
  fCheckCompareTime.OnClick := CheckboxClicked;
  fCheckCompareLowPrecision := AddCheckbox(9, 9, GetMsg(MCompareLowPrecision), False);
  fCheckIgnoreTimeZone := AddCheckbox(9, 10, GetMsg(MCompareIgnoreTimeZone), False);
  fCheckCompareSize := AddCheckbox(5, 11, GetMsg(MCompareSize), False);
  fCheckCompareContents := AddCheckbox(5, 12, GetMsg(MCompareContents), False);
  fCheckCompareContents.OnClick := CheckboxClicked;
  fCheckCompareContentsIgnore := AddCheckbox(9, 13, GetMsg(MCompareContentsIgnore), False);
  fCheckCompareContentsIgnore.OnClick := CheckboxClicked;
  fRadioIgnoreNewLines := AddRadioButton(13, 14, GetMsg(MCompareIgnoreNewLines), False);
  fRadioIgnoreWhiteSpace := AddRadioButton(13, 15, GetMsg(MCompareIgnoreWhitespace), False);
  fSeparator2 := AddSeparator(16);
  fCheckSelectOnBothPanels := AddCheckbox(5, 17, GetMsg(MSelectOnBothPanels), False);
  fSeparator3 := AddSeparator(18);
  fButtonOK := AddButton(0, 19, GetMsg(MOk));
  fButtonOK.Default := True;
  fButtonCancel := AddButton(0, 19, GetMsg(MCancel));
end;

destructor TDiscoverDifferencesDialog.Destroy;
begin
  inherited;
end;

function TDiscoverDifferencesDialog.DialogProc(Msg, Param1, Param2: TIntPtr; var DlgProcResult: TIntPtr): boolean;
begin
  Result := False;
  case Msg of
    DN_BTNCLICK:
      Result := HandleBTNCLICK(Param1, Param2, DlgProcResult);
  end;
  if not Result then
    Result := inherited DialogProc(Msg, Param1, Param2, DlgProcResult);
end;

procedure TDiscoverDifferencesDialog.DialogToOptions;
begin
  Options.ProcessSubfolders := CheckProcessSubfolders.Enabled and CheckProcessSubfolders.Checked;
  Options.UseMaxScanDepth := Options.ProcessSubfolders and CheckUseMaxScanDepth.Enabled and CheckUseMaxScanDepth.Checked;
  Options.MaxScanDepth := StrToIntDef(EditMaxScanDepth.Text, Options.MaxScanDepth);
  Options.ProcessSelected := CheckProcessSelected.Enabled and CheckProcessSelected.Checked;
  Options.CompareTime := CheckCompareTime.Enabled and CheckCompareTime.Checked;
  Options.LowPrecisionTime := Options.CompareTime and CheckCompareLowPrecision.Enabled and CheckCompareLowPrecision.Checked;
  Options.IgnorePossibleTimeZoneDifferences := Options.CompareTime and CheckIgnoreTimeZone.Enabled and CheckIgnoreTimeZone.Checked;
  Options.CompareSize := CheckCompareSize.Enabled and CheckCompareSize.Checked;
  Options.CompareContents := CheckCompareContents.Enabled and CheckCompareContents.Checked;
  Options.CompareContentsIgnore := Options.CompareContents and CheckCompareContentsIgnore.Enabled and CheckCompareContentsIgnore.Checked;
  Options.IgnoreNewLines := Options.CompareContentsIgnore and RadioIgnoreNewLines.Enabled and RadioIgnoreNewLines.Checked;
  Options.IgnoreWhitespace := Options.CompareContentsIgnore and RadioIgnoreWhiteSpace.Enabled and RadioIgnoreWhiteSpace.Checked;
  Options.SelectDifferencesOnBothPanels := CheckSelectOnBothPanels.Enabled and CheckSelectOnBothPanels.Checked;
end;

function TDiscoverDifferencesDialog.Execute: boolean;
var
  Button: TFarDialog.TButton;
begin
  Result := False;
  Resize(Screen.Width, Screen.Height);
  OptionsToDialog;
  if BuildDialog(Width, Height, 0) then begin
    if inherited Execute(Button) then begin
      if Button = ButtonOK then begin
        DialogToOptions;
        Result := True;
      end;
    end;
  end;
end;

procedure TDiscoverDifferencesDialog.OptionsToDialog;
begin
  CheckProcessSubfolders.Checked := Options.ProcessSubfolders;
  CheckUseMaxScanDepth.Checked := Options.UseMaxScanDepth;
  EditMaxScanDepth.Text := IntToStr(Options.MaxScanDepth);
  CheckProcessSelected.Checked := LeftPanel.HasSelectedItems or RightPanel.HasSelectedItems;
  CheckCompareTime.Checked := Options.CompareTime;
  CheckCompareLowPrecision.Checked := Options.LowPrecisionTime;
  CheckIgnoreTimeZone.Checked := Options.IgnorePossibleTimeZoneDifferences;
  CheckCompareSize.Checked := Options.CompareSize;
  CheckCompareContents.Checked := Options.CompareContents;
  CheckCompareContentsIgnore.Checked := Options.CompareContentsIgnore;
  RadioIgnoreNewLines.Checked := Options.IgnoreNewLines or (not Options.IgnoreWhitespace);
  RadioIgnoreWhiteSpace.Checked := Options.IgnoreWhitespace and (not Options.IgnoreNewLines);
  CheckSelectOnBothPanels.Checked := Options.SelectDifferencesOnBothPanels;
  UpdateControls(nil, csUnchecked);
end;

procedure TDiscoverDifferencesDialog.UpdateControls(ActiveCheckbox: TFarDialog.TCheckbox; ActiveCheckboxFinalState: TFarDialog.TCheckbox.TCheckState);

  function IsChecked(Checkbox: TFarDialog.TCheckbox): boolean;
  begin
    if ActiveCheckbox = Checkbox then
      Result := ActiveCheckboxFinalState = csChecked
    else
      Result := Checkbox.Checked;
  end;

begin
  CheckProcessSubfolders.Enabled := (not LeftPanel.IsPluginPanel) and (not RightPanel.IsPluginPanel);
  CheckUseMaxScanDepth.Enabled := CheckProcessSubfolders.Enabled and IsChecked(CheckProcessSubfolders);
  EditMaxScanDepth.Enabled := CheckUseMaxScanDepth.Enabled and IsChecked(CheckUseMaxScanDepth);
  CheckProcessSelected.Enabled := LeftPanel.HasSelectedItems or RightPanel.HasSelectedItems;
  CheckCompareLowPrecision.Enabled := CheckCompareTime.Enabled and IsChecked(CheckCompareTime);
  CheckIgnoreTimeZone.Enabled := CheckCompareTime.Enabled and IsChecked(CheckCompareTime);
  CheckCompareContents.Enabled := (not LeftPanel.IsPluginPanel) and (not RightPanel.IsPluginPanel);
  CheckCompareContentsIgnore.Enabled := CheckCompareContents.Enabled and IsChecked(CheckCompareContents);
  RadioIgnoreNewLines.Enabled := CheckCompareContentsIgnore.Enabled and IsChecked(CheckCompareContentsIgnore);
  RadioIgnoreWhiteSpace.Enabled := CheckCompareContentsIgnore.Enabled and IsChecked(CheckCompareContentsIgnore);
end;

end.
