unit uSynchronizeDirectoriesDialog;
{$INCLUDE 'DirSync3.inc'}

interface

uses
  SysUtils, Windows, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uSystem,
  uDirSyncConsts, uMessages, uOptions;

type

  TSynchronizeDirectoriesDialog = class(TFarDialog)
  public
    type
      TDifferentFilesAction = (dfaAsk, dfaSkip, dfaCopyToLeft, dfaCopyToRight);
      TSynchronizeDirectoriesAction = (sdaSynchronize, sdaReturnToEditor, sdaCancel);
  private
    fCopyToLeftPanel: boolean;
    fConfirmOverwriteLeft: boolean;
    fCopyToRightPanel: boolean;
    fConfirmOverwriteRight: boolean;
    fDeleteLeftMissingOnRight: boolean;
    fDeleteRightMissingOnLeft: boolean;
    fDiffTypesPresent: array[TDifferenceType] of boolean;
    fDifferentFilesAction: TDifferentFilesAction;
    fSilentMode: boolean;
    fGlobalRemember: boolean;
    function GetDiffTypePresent(DiffType: TDifferenceType): boolean;
    procedure SetDiffTypePresent(DiffType: TDifferenceType; const Value: boolean);
  private
    fFrame: TFarDialog.TDoubleBox;
    fLabelBothFilesExist: TFarDialog.TLabel;
    fCheckCopyToLeft: TFarDialog.TCheckbox;
    fCheckCopyToLeftOverwrite: TFarDialog.TCheckbox;
    fCheckCopyToRight: TFarDialog.TCheckbox;
    fCheckCopyToRightOverwrite: TFarDialog.TCheckbox;
    fLabelOneFileDoesNotExist: TFarDialog.TLabel;
    fCheckDeleteLeft: TFarDialog.TCheckbox;
    fCheckDeleteRight: TFarDialog.TCheckbox;
    fLabelBothFilesExistWithSameDate: TFarDialog.TLabel;
    fRadioDifferentFilesAsk: TFarDialog.TRadioButton;
    fRadioDifferentFilesOverwriteLeft: TFarDialog.TRadioButton;
    fRadioDifferentFilesOverwriteRight: TFarDialog.TRadioButton;
    fRadioDifferentFilesSkip: TFarDialog.TRadioButton;
    fSeparator1: TFarDialog.TSeparator;
    fCheckSilentMode: TFarDialog.TCheckbox;
    fCheckGlobalRemember: TFarDialog.TCheckbox;
    fSeparator2: TFarDialog.TSeparator;
    fButtonOK: TFarDialog.TButton;
    fButtonCancel: TFarDialog.TButton;
    fButtonEditor: TFarDialog.TButton;
    fWidth: integer;
    fHeight: integer;
  protected
    procedure UpdateControls(ActiveCheckbox: TFarDialog.TCheckbox; ActiveCheckboxFinalState: TFarDialog.TCheckbox.TCheckState);
    property Frame: TFarDialog.TDoubleBox read fFrame;
    property LabelBothFilesExist: TFarDialog.TLabel read fLabelBothFilesExist;
    property CheckCopyToLeft: TFarDialog.TCheckbox read fCheckCopyToLeft;
    property CheckCopyToLeftOverwrite: TFarDialog.TCheckbox read fCheckCopyToLeftOverwrite;
    property CheckCopyToRight: TFarDialog.TCheckbox read fCheckCopyToRight;
    property CheckCopyToRightOverwrite: TFarDialog.TCheckbox read fCheckCopyToRightOverwrite;
    property LabelOneFileDoesNotExist: TFarDialog.TLabel read fLabelOneFileDoesNotExist;
    property CheckDeleteLeft: TFarDialog.TCheckbox read fCheckDeleteLeft;
    property CheckDeleteRight: TFarDialog.TCheckbox read fCheckDeleteRight;
    property LabelBothFilesExistWithSameDate: TFarDialog.TLabel read fLabelBothFilesExistWithSameDate;
    property RadioDifferentFilesAsk: TFarDialog.TRadioButton read fRadioDifferentFilesAsk;
    property RadioDifferentFilesOverwriteLeft: TFarDialog.TRadioButton read fRadioDifferentFilesOverwriteLeft;
    property RadioDifferentFilesOverwriteRight: TFarDialog.TRadioButton read fRadioDifferentFilesOverwriteRight;
    property RadioDifferentFilesSkip: TFarDialog.TRadioButton read fRadioDifferentFilesSkip;
    property Separator1: TFarDialog.TSeparator read fSeparator1;
    property CheckSilentMode: TFarDialog.TCheckbox read fCheckSilentMode;
    property CheckGlobalRemember: TFarDialog.TCheckbox read fCheckGlobalRemember;
    property Separator2: TFarDialog.TSeparator read fSeparator2;
    property ButtonOK: TFarDialog.TButton read fButtonOK;
    property ButtonCancel: TFarDialog.TButton read fButtonCancel;
    property ButtonEditor: TFarDialog.TButton read fButtonEditor;
    property Width: integer read fWidth;
    property Height: integer read fHeight;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; override;
    function Execute: TSynchronizeDirectoriesAction;
    procedure SetDefaultSettings;
    property CopyToLeftPanel: boolean read fCopyToLeftPanel write fCopyToLeftPanel;
    property ConfirmOverwriteLeft: boolean read fConfirmOverwriteLeft write fConfirmOverwriteLeft;
    property CopyToRightPanel: boolean read fCopyToRightPanel write fCopyToRightPanel;
    property ConfirmOverwriteRight: boolean read fConfirmOverwriteRight write fConfirmOverwriteRight;
    property DeleteLeftMissingOnRight: boolean read fDeleteLeftMissingOnRight write fDeleteLeftMissingOnRight;
    property DeleteRightMissingOnLeft: boolean read fDeleteRightMissingOnLeft write fDeleteRightMissingOnLeft;
    property DifferentFilesAction: TDifferentFilesAction read fDifferentFilesAction write fDifferentFilesAction;
    property DiffTypePresent[DiffType: TDifferenceType]: boolean read GetDiffTypePresent write SetDiffTypePresent;
    property SilentMode: boolean read fSilentMode write fSilentMode;
    property GlobalRemember: boolean read fGlobalRemember write fGlobalRemember;
  end;

implementation

{ TDirSyncSyncDialog }

procedure TSynchronizeDirectoriesDialog.Clear;
begin
  inherited;
  fFrame := nil;
  fLabelBothFilesExist := nil;
  fCheckCopyToLeft := nil;
  fCheckCopyToLeftOverwrite := nil;
  fCheckCopyToRight := nil;
  fCheckCopyToRightOverwrite := nil;
  fLabelBothFilesExistWithSameDate := nil;
  fRadioDifferentFilesAsk := nil;
  fRadioDifferentFilesOverwriteLeft := nil;
  fRadioDifferentFilesOverwriteRight := nil;
  fRadioDifferentFilesSkip := nil;
  fLabelOneFileDoesNotExist := nil;
  fCheckDeleteLeft := nil;
  fCheckDeleteRight := nil;
  fSeparator1 := nil;
  fCheckSilentMode := nil;
  fCheckGlobalRemember := nil;
  fSeparator2 := nil;
  fButtonOK := nil;
  fButtonEditor := nil;
  fButtonCancel := nil;
end;

constructor TSynchronizeDirectoriesDialog.Create;
begin
  inherited Create(DIALOG_MAIN_GUID);
  SetDefaultSettings;
  fWidth := 74;
  fHeight := 19;
  fFrame := AddDoubleBox(3, 1, fWidth-6, fHeight-2, GetMsg(MTitle));
  fLabelBothFilesExist := AddLabel(5, 2, 0, GetMsg(MBothFilesExist));
  fCheckCopyToLeft := AddCheckbox(5, 3, GetMsg(MCopyRightToLeft), False);
  fCheckCopyToLeftOverwrite := AddCheckbox(8, 4, GetMsg(MOverwriteRightToLeft), False);
  fCheckCopyToRight := AddCheckbox(5, 5, GetMsg(MCopyLeftToRight), False);
  fCheckCopyToRightOverwrite := AddCheckbox(8, 6, GetMsg(MOverwriteLeftToRight), False);
  fLabelBothFilesExistWithSameDate := AddLabel(5, 7, 0, GetMsg(MBothFilesExistSameDate));
  fRadioDifferentFilesAsk := AddRadioButton(0, 8, GetMsg(MAskUser), True);
  fRadioDifferentFilesAsk.CenterGroup := True;
  fRadioDifferentFilesSkip := AddRadioButton(0, 8, GetMsg(MSkip), False);
  fRadioDifferentFilesSkip.CenterGroup := True;
  fRadioDifferentFilesOverwriteLeft := AddRadioButton(0, 8, GetMsg(MOverwriteLeft2), False);
  fRadioDifferentFilesOverwriteLeft.CenterGroup := True;
  fRadioDifferentFilesOverwriteRight := AddRadioButton(0, 8, GetMsg(MOverwriteRight2), False);
  fRadioDifferentFilesOverwriteRight.CenterGroup := True;
  fLabelOneFileDoesNotExist := AddLabel(5, 9, 0, GetMsg(MEraseAbsentFiles));
  fCheckDeleteLeft := AddCheckbox(5, 10, GetMsg(MEraseAbsentFilesLeft), False);
  fCheckDeleteRight := AddCheckbox(5, 11, GetMsg(MEraseAbsentFilesRight), False);
  fSeparator1 := AddSeparator(12);
  fCheckSilentMode := AddCheckbox(5, 13, GetMsg(MSilentMode), False);
  fCheckGlobalRemember := AddCheckbox (5, 14, GetMsg(MGlobalRemember), False);
  fSeparator2 := AddSeparator(15);
  fButtonOK := AddButton(0, 16, GetMsg(MOk));
  fButtonOK.Default := True;
  fButtonEditor := AddButton(0, 16, GetMsg(MReturnToEditor));
  fButtonCancel := AddButton(0, 16, GetMsg(MCancel));
end;

destructor TSynchronizeDirectoriesDialog.Destroy;
begin
  inherited;
end;

function TSynchronizeDirectoriesDialog.Execute: TSynchronizeDirectoriesAction;
const
  EnabledFlag: array[boolean] of DWORD = (DIF_DISABLE, 0);
var
  Button: TFarDialog.TButton;
begin
  Result := sdaCancel;
  UpdateControls(nil, csUnchecked);
  CheckCopyToLeft.Checked := CopyToLeftPanel and CheckCopyToLeft.Enabled;
  CheckCopyToLeftOverwrite.Checked := ConfirmOverwriteLeft and CheckCopyToLeftOverwrite.Enabled;
  CheckCopyToRight.Checked := CopyToRightPanel and CheckCopyToRight.Enabled;
  CheckCopyToRightOverwrite.Checked := ConfirmOverwriteRight and CheckCopyToRightOverwrite.Enabled;
  RadioDifferentFilesAsk.Checked := True;
  CheckDeleteLeft.Checked := DeleteLeftMissingOnRight and CheckDeleteLeft.Enabled;
  CheckDeleteRight.Checked := DeleteRightMissingOnLeft and CheckDeleteRight.Enabled;
  CheckSilentMode.Checked := SilentMode;
  CheckGlobalRemember.Checked := GlobalRemember;
  if BuildDialog(Width, Height, 0) then begin
    if inherited Execute(Button) then begin
      if Button = ButtonOK then begin
        //Log('OK pressed, reading the settings');
        CopyToLeftPanel := CheckCopyToLeft.Checked;
        ConfirmOverwriteLeft := CheckCopyToLeftOverwrite.Checked;
        CopyToRightPanel := CheckCopyToRight.Checked;
        ConfirmOverwriteRight := CheckCopyToRightOverwrite.Checked;
        DeleteLeftMissingOnRight := CheckDeleteLeft.Checked;
        DeleteRightMissingOnLeft := CheckDeleteRight.Checked;
        if RadioDifferentFilesAsk.Checked then
          DifferentFilesAction := dfaAsk
        else if RadioDifferentFilesOverwriteLeft.Checked then
          DifferentFilesAction := dfaCopyToLeft
        else if RadioDifferentFilesOverwriteRight.Checked then
          DifferentFilesAction := dfaCopyToRight
        else if RadioDifferentFilesSkip.Checked then
          DifferentFilesAction := dfaSkip
        else
          DifferentFilesAction := dfaAsk;
        fSilentMode := CheckSilentMode.Checked;
        fGlobalRemember := CheckGlobalRemember.Checked;
        Result := sdaSynchronize;
      end
      else if Button = ButtonEditor then begin
        Result := sdaReturnToEditor;
      end;
    end;
  end;
end;

function TSynchronizeDirectoriesDialog.GetDiffTypePresent(DiffType: TDifferenceType): boolean;
begin
  Result := fDiffTypesPresent[DiffType];
end;

procedure TSynchronizeDirectoriesDialog.SetDefaultSettings;
begin
  CopyToLeftPanel := True;
  ConfirmOverwriteLeft := True;
  CopyToRightPanel := True;
  ConfirmOverwriteRight := True;
  DeleteLeftMissingOnRight := False;
  DeleteRightMissingOnLeft := False;
  DifferentFilesAction := dfaAsk;
  SilentMode := False;
  GlobalRemember := False;
end;

procedure TSynchronizeDirectoriesDialog.SetDiffTypePresent(DiffType: TDifferenceType; const Value: boolean);
begin
  fDiffTypesPresent[DiffType] := Value;
end;

procedure TSynchronizeDirectoriesDialog.UpdateControls(ActiveCheckbox: TFarDialog.TCheckbox; ActiveCheckboxFinalState: TFarDialog.TCheckbox.TCheckState);
begin
  CheckCopyToLeft.Enabled := DiffTypePresent[dtMissingLeft] or DiffTypePresent[dtOlderLeft];
  CheckCopyToLeftOverwrite.Enabled := DiffTypePresent[dtMissingLeft] or DiffTypePresent[dtOlderLeft];
  CheckCopyToRight.Enabled := DiffTypePresent[dtMissingRight] or DiffTypePresent[dtOlderRight];
  CheckCopyToRightOverwrite.Enabled := DiffTypePresent[dtMissingRight] or DiffTypePresent[dtOlderRight];
  RadioDifferentFilesAsk.Enabled := DiffTypePresent[dtDifferent];
  RadioDifferentFilesSkip.Enabled := DiffTypePresent[dtDifferent];
  RadioDifferentFilesOverwriteLeft.Enabled := DiffTypePresent[dtDifferent];
  RadioDifferentFilesOverwriteRight.Enabled := DiffTypePresent[dtDifferent];
  CheckDeleteLeft.Enabled := DiffTypePresent[dtMissingRight];
  CheckDeleteRight.Enabled := DiffTypePresent[dtMissingLeft];
end;

end.
